package com.mopub.mobileads;

import android.content.Context;
import android.text.TextUtils;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.mopub.common.Preconditions;
import com.mopub.common.logging.MoPubLog;
import com.mopub.nativeads.BaseNativeAd;
import com.mopub.nativeads.MoPubAdRenderer;
import com.mopub.nativeads.NativeImageHelper;
import com.mopub.nativeads.NativeRendererHelper;

import java.util.ArrayList;
import java.util.List;
import java.util.WeakHashMap;

public class AlxNativeAdRenderer implements MoPubAdRenderer<AlxNativeAdapter.AlgorixNativeAd> {
    private static final String TAG = "AlxNativeAdRenderer";

    private final AlxNativeAdViewBinder mViewBinder;
    private final WeakHashMap<View, AlxNativeViewHolder> mViewHolderMap;

    public AlxNativeAdRenderer(AlxNativeAdViewBinder viewBinder) {
        mViewBinder = viewBinder;
        mViewHolderMap = new WeakHashMap<View, AlxNativeViewHolder>();
    }

    @NonNull
    @Override
    public View createAdView(@NonNull Context context, @Nullable ViewGroup parent) {
        Preconditions.checkNotNull(context);
        return LayoutInflater.from(context).inflate(mViewBinder.layoutId, parent, false);
    }

    @Override
    public void renderAdView(@NonNull View view, @NonNull AlxNativeAdapter.AlgorixNativeAd algorixNativeAd) {
        Preconditions.checkNotNull(view);
        Preconditions.checkNotNull(algorixNativeAd);

        AlxNativeViewHolder holder = mViewHolderMap.get(view);
        if (holder == null) {
            holder = AlxNativeViewHolder.fromViewBinder(view, mViewBinder);
            mViewHolderMap.put(view, holder);
        }
        update(holder, algorixNativeAd);
    }

    @Override
    public boolean supports(@NonNull BaseNativeAd nativeAd) {
        Preconditions.checkNotNull(nativeAd);
        return nativeAd instanceof AlxNativeAdapter.AlgorixNativeAd;
    }

    private void update(final AlxNativeViewHolder holder, final AlxNativeAdapter.AlgorixNativeAd nativeAd) {
        if (nativeAd == null) {
            return;
        }

        NativeRendererHelper.addTextView(holder.getTitleView(), nativeAd.getTitle());
        NativeRendererHelper.addTextView(holder.getTextView(), nativeAd.getText());
        NativeRendererHelper.addTextView(holder.getCallToActionView(), nativeAd.getCallToAction());

        if (!TextUtils.isEmpty(nativeAd.getIconImageUrl()) && holder.getIconView() != null) {
            NativeImageHelper.loadImageView(nativeAd.getIconImageUrl(), holder.getIconView());
        }

        if (!TextUtils.isEmpty(nativeAd.getMainImageUrl()) && holder.getMainImageView() != null) {
            NativeImageHelper.loadImageView(nativeAd.getMainImageUrl(), holder.getMainImageView());
        }

        if (nativeAd.getAdLogo() != null && holder.logoView != null) {
            holder.logoView.setImageBitmap(nativeAd.getAdLogo());
        }

        List<View> clickList = new ArrayList<>();
        clickList.add(holder.iconView);
        clickList.add(holder.mainImageView);
        clickList.add(holder.titleView);
        clickList.add(holder.textView);
        clickList.add(holder.callToActionView);
        nativeAd.registerViewForInteraction((ViewGroup) holder.mainView, clickList);
    }

    private static class AlxNativeViewHolder {
        private View mainView;
        private TextView titleView;
        private TextView textView;
        private TextView callToActionView;
        private ImageView iconView;
        private ImageView mainImageView;
        private ImageView logoView;

        private AlxNativeViewHolder() {
        }

        static AlxNativeViewHolder fromViewBinder(final View view, final AlxNativeAdViewBinder viewBinder) {
            if (view == null || viewBinder == null) {
                return new AlxNativeViewHolder();
            }
            try {
                AlxNativeViewHolder viewHolder = new AlxNativeViewHolder();
                viewHolder.mainView = view;
                viewHolder.titleView = view.findViewById(viewBinder.titleId);
                viewHolder.textView = view.findViewById(viewBinder.textId);
                viewHolder.callToActionView = view.findViewById(viewBinder.callToActionId);
                viewHolder.iconView = view.findViewById(viewBinder.iconViewId);
                viewHolder.mainImageView = view.findViewById(viewBinder.mainImageViewId);
                viewHolder.logoView = view.findViewById(viewBinder.logoViewId);
                return viewHolder;
            } catch (Exception e) {
                MoPubLog.log(MoPubLog.SdkLogEvent.CUSTOM, "Could not cast from id in AlxNativeViewHolder to expected View type",
                        e);
                return new AlxNativeViewHolder();
            }
        }

        public View getMainView() {
            return mainView;
        }

        public TextView getTitleView() {
            return titleView;
        }

        public TextView getTextView() {
            return textView;
        }

        public TextView getCallToActionView() {
            return callToActionView;
        }

        public ImageView getIconView() {
            return iconView;
        }

        public ImageView getMainImageView() {
            return mainImageView;
        }
    }


}